@@ -30,7 +30,6 @@
 #include <pwd.h>
 #include <unistd.h>
 #include <errno.h>
-#include <unistd.h>
 #include <sys/types.h>
 #ifdef HAVE_UTMPX_H
 #include <utmpx.h>
@@ -52,8 +51,6 @@
 #define PATH_FALSE "/bin/false"
 #define PATH_GDM_CUSTOM "/etc/gdm/custom.conf"
 
-#define USERDIR LOCALSTATEDIR "/lib/AccountsService/users"
-
 static const char *default_excludes[] = {
         "bin",
         "root",
@@ -161,30 +158,47 @@ error_get_type (void)
 gboolean
 daemon_local_user_is_excluded (Daemon *daemon, const gchar *username, const gchar *shell)
 {
-        char *basename, *nologin_basename, *false_basename;
         int ret;
 
-        if (shell != NULL && shell[0] == '\0') {
+        if (g_hash_table_lookup (daemon->priv->exclusions, username)) {
                 return TRUE;
         }
 
         ret = FALSE;
-        basename = g_path_get_basename (shell);
-        nologin_basename = g_path_get_basename (PATH_NOLOGIN);
-        false_basename = g_path_get_basename (PATH_FALSE);
 
-        if (g_strcmp0 (basename, nologin_basename) == 0) {
-                ret = TRUE;
-        } else if (g_strcmp0 (basename, false_basename) == 0) {
-                ret = TRUE;
-        } else if (g_hash_table_lookup (daemon->priv->exclusions, username)) {
+        if (shell != NULL) {
+                char *basename, *nologin_basename, *false_basename;
+
+#ifdef HAVE_GETUSERSHELL
+                char *valid_shell;
+
                 ret = TRUE;
+                setusershell ();
+                while ((valid_shell = getusershell ()) != NULL) {
+                        if (g_strcmp0 (shell, valid_shell) != 0)
+                                continue;
+                        ret = FALSE;
+                }
+                endusershell ();
+#endif
+
+                basename = g_path_get_basename (shell);
+                nologin_basename = g_path_get_basename (PATH_NOLOGIN);
+                false_basename = g_path_get_basename (PATH_FALSE);
+
+                if (shell[0] == '\0') {
+                        ret = TRUE;
+                } else if (g_strcmp0 (basename, nologin_basename) == 0) {
+                        ret = TRUE;
+                } else if (g_strcmp0 (basename, false_basename) == 0) {
+                        ret = TRUE;
+                }
+
+                g_free (basename);
+                g_free (nologin_basename);
+                g_free (false_basename);
         }
 
-        g_free (basename);
-        g_free (nologin_basename);
-        g_free (false_basename);
-
         return ret;
 }
 
@@ -286,32 +300,22 @@ reload_passwd (Daemon *daemon)
         GSList *old_users;
         GSList *new_users;
         GSList *list;
-#ifdef HAVE_FGETPWENT
         FILE *fp;
-#endif
         User *user = NULL;
 
         old_users = NULL;
         new_users = NULL;
 
-#ifdef HAVE_FGETPWENT
         errno = 0;
         fp = fopen (PATH_PASSWD, "r");
         if (fp == NULL) {
                 g_warning ("Unable to open %s: %s", PATH_PASSWD, g_strerror (errno));
                 goto out;
         }
-#else
-        setpwent();
-#endif
         g_hash_table_foreach (daemon->priv->users, listify_hash_values_hfunc, &old_users);
         g_slist_foreach (old_users, (GFunc) g_object_ref, NULL);
 
-#ifdef HAVE_FGETPWENT
         while ((pwent = fgetpwent (fp)) != NULL) {
-#else
-        while ((pwent = getpwent ()) != NULL) {
-#endif
                 /* Skip system users... */
                 if (daemon_local_user_is_excluded (daemon, pwent->pw_name, pwent->pw_shell)) {
                         g_debug ("skipping user: %s", pwent->pw_name);
@@ -361,12 +365,10 @@ reload_passwd (Daemon *daemon)
                 }
         }
 
-#ifdef HAVE_FGETPWENT
  out:
         /* Cleanup */
 
         fclose (fp);
-#endif
 
         g_slist_foreach (new_users, (GFunc) g_object_thaw_notify, NULL);
         g_slist_foreach (new_users, (GFunc) g_object_unref, NULL);
@@ -835,16 +837,27 @@ finish_list_cached_users (gpointer user_data)
         const gchar *name;
         User *user;
         uid_t uid;
+        const gchar *shell;
 
         object_paths = g_ptr_array_new ();
 
         g_hash_table_iter_init (&iter, data->daemon->priv->users);
         while (g_hash_table_iter_next (&iter, (gpointer *)&name, (gpointer *)&user)) {
                 uid = user_local_get_uid (user);
-                if (!daemon_local_user_is_excluded (data->daemon, name, NULL)) {
-                        g_debug ("user %s %ld not excluded\n", name, (long) uid);
-                        g_ptr_array_add (object_paths, (gpointer) user_local_get_object_path (user));
+                shell = user_local_get_shell (user);
+
+                if (user_local_get_system_account (user)) {
+                        g_debug ("user %s %ld is system account, so excluded\n", name, (long) uid);
+                        continue;
                 }
+
+                if (daemon_local_user_is_excluded (data->daemon, name, shell)) {
+                        g_debug ("user %s %ld excluded\n", name, (long) uid);
+                        continue;
+                }
+
+                g_debug ("user %s %ld not excluded\n", name, (long) uid);
+                g_ptr_array_add (object_paths, (gpointer) user_local_get_object_path (user));
         }
         g_ptr_array_add (object_paths, NULL);
 
@@ -1030,7 +1043,10 @@ daemon_delete_user_authorized_cb (Daemon                *daemon,
 
         filename = g_build_filename (USERDIR, pwent->pw_name, NULL);
         g_remove (filename);
+        g_free (filename);
 
+        filename = g_build_filename (ICONDIR, pwent->pw_name, NULL);
+        g_remove (filename);
         g_free (filename);
 
         accounts_accounts_complete_delete_user (NULL, context);
